@@ -1,14 +1,11 @@
 import stat
 import requests
 import tempfile
-import logging
 import subprocess
 from pathlib import Path
 
-logger = logging.getLogger(__name__)
-
 class Megatools():
-    def __init__(self, executable=None, callback=None):
+    def __init__(self, executable=None):
         if not executable:
             executable = Path(tempfile.gettempdir()) / "megatools"
             if not executable.exists():
@@ -17,29 +14,22 @@ class Megatools():
                     f.write(binary.content)
                 executable.chmod(executable.stat().st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
         self.executable = str(executable)
-        self.callback = callback
-    
-    def _callback(self, proc):
-        output = proc.stdout.readline()
-        if output:
-            print(output.decode())
 
-    def _execute(self, cmd):
+    def _execute(self, cmd, callback, callback_args):
         proc = subprocess.Popen(
             cmd,
             stdout=subprocess.PIPE, 
             stderr=subprocess.PIPE
-            )
+        )
         while proc.poll() is None:
-            if self.callback:
-                self.callback(proc)
-            else:
-                self._callback(proc)
+            callback(proc, *callback_args)
         return (proc.stdout, proc.returncode)
 
     def download(
         self,
         link,
+        callback=lambda proc: print(proc.stdout.readline().decode('utf-8', 'ignore')),
+        callback_args=(),
         **options
     ):
         """
@@ -57,7 +47,5 @@ class Megatools():
         cmd = [self.executable, "dl", link, "--no-ask-password"]
         for option, value in options:
             cmd.append(f"--{option}={value}")
-        logger.info(cmd)
-        stdout, exit_code = self._execute(cmd)
-        logger.info(exit_code)
-        return stdout
+        stdout, exit_code = self._execute(cmd, callback, callback_args)
+        return (stdout, exit_code)
\ No newline at end of file
